/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2003 Ashley Montanaro and Daniel Winter.
	Affix stack fixes: Andrei Emeltchenko and Dmitri Kasatkin.
	
	2008: some ugly hack by michael vogt - www.neophob.com
	
*/

#include <cstdlib>
#include <iostream>
#include <fstream>
#include <cerrno>
#include <csignal>

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>

#include <netinet/in.h>
#include <arpa/inet.h>
#include <resolv.h>
#include <netdb.h>
#include <sys/socket.h>

#include <bluetooth/bluetooth.h>
#include <bluetooth/rfcomm.h>

#include "mpdctrl.h"

#include "BemusedServerDlg.h"

//#define debuginf

using namespace std;

/* Test modes */
enum {
    SEND,
    RECV,
    RECONNECT,
    MULTY,
    DUMP,
    CONNECT,
    CRECV,
    LSEND
};

const static char bookpath[] = "Bookmarks/";
char wbookpath[MAX_PATH];		// has any paths added or removed

//affix support removed...
bdaddr_t bdaddr;

/////////////////////////////////////////////////////////////////////////////
// CDirTreeNode class

CDirTreeNode::CDirTreeNode()
{
    iFirstChild = NULL;
    iNextSibling = NULL;
    iParent = NULL;
    iFileName = NULL;
    iNodeType = iFileType = 0x0;
    iWrittenToBuffer = false;
}

CDirTreeNode::~CDirTreeNode()
{
    if(iFirstChild)    
        delete iFirstChild;
    if(iNextSibling)
        delete iNextSibling;
    if(iFileName)	
        delete iFileName;
}

/////////////////////////////////////////////////////////////////////////////
// CBemusedServerDlg dialog

CBemusedServerDlg::CBemusedServerDlg()
{
    iDirTreeRoot = iDirTreeCurrentNode = NULL;
    lastTime = 0;
}

CBemusedServerDlg::~CBemusedServerDlg()
{
    cout << "Exit Application\n";
    delete iDirTreeRoot;
    close(iCommHandle);
    close(socket1);
    close_mpd_connection();
}

void SignalHandler(int signal)
{
    printf("\n## SignalHandler: got signal %d, exit Application\n", signal);    
    close_mpd_connection();
    exit(0);
}
	    
bool CBemusedServerDlg::OnInitDialog(bool daemonize)
{
    int exitCheck;
    Log("Welcome to Bemused 1.73 + lastest CVS fixes\n");
    Log("Copyright Ashley Montanaro 2003\n");
    Log("Linux port from Daniel Winter\n");
    Log("MPD Hack + various fixes by Michael Vogt 2008\n\n");

    ReadIniFile();
#ifdef debuginf
    Log("ini file looks good..\n");
#endif

    signal(SIGHUP, SignalHandler);
    signal(SIGINT, SignalHandler);
    signal(SIGQUIT, SignalHandler);
    signal(SIGTERM, SignalHandler);
    
    socket1 = BluetoothConnectToSocket(channel);
#ifdef debuginf
    Log("Bluetooth is connected to socket..\n");
#endif

    CheckWinamp("Blub");

    if (daemonize) {
	Log("daemonizing\n");
	if (daemon(0, 1) < 0) {
	    perror("daemon");
	    exit(1);
	}
	fclose(stdout);
    }

    while (1) {
	iCommHandle = BluetoothWaitforConnect(socket1);
	exitCheck = HandleBluetooth();
	if (exitCheck==1)
	    return true;
    }

#ifdef debuginf
    Log("OnInitDialog, return true now\n");
#endif
    
    return true;
}


void CBemusedServerDlg::ReadIniFile()
{
    bool mp3set = false, autoset = false, xmmsdirset = false, fastforbackspeedset = false;
    bool channelset = false, bookmarkset = false, bookmarkread = false, portset = false;
    char row[1025], wert[1025], einstellung[1025];
    char *tmpStr;
    tcpport = 7654;
    channel = 10;

    tcp = false;
    
    strcpy(winampPath, "mpd");
//    startAutomatically = 1;
    ifstream conffile("/etc/bemused.conf");
    if (!conffile) {	
	Log("Could not read the configfile /etc/bemused.conf\n");
	exit(2);
    }
/*
    no need to check size of string to copy, as the value can be max. 1024 bytes, our variable is 1024.
*/

    Log("[bemusedMPD] Read config file /etc/bemused.conf\n");
    while (conffile.getline(row, 1024)) {
	if (!(row[0] == '#') && !(row[0] == '\0') && (strchr(row, '=')!=NULL) ) {

            /* split up string, einstellung = wert */
            tmpStr=strtok(row, "=");
	    if (tmpStr == NULL) {
	            Log("[bemusedMPD] Invalid config entry, Line: %s\n", row);
	            goto nextConfigLine;
	    }
	    strcpy(einstellung, tmpStr);

	    tmpStr=strtok(NULL, "=");
	    if (tmpStr == NULL) {
	            Log("[bemusedMPD] Invalid config entry, Line: %s\n", row);
	            goto nextConfigLine;
	    }	    	    
	    strcpy(wert, tmpStr);
	    
	    if (!strcmp(einstellung, "mp3dir")) {
		strcpy(mp3Dirs, wert);
		mp3set = true;
	    }
	    if (!strcmp(einstellung, "tcp")) {
		int tmptcp;
		tmptcp = atoi(wert);
		if (tmptcp == 1)
		    tcp = true;
	    }
	    if (!strcmp(einstellung, "channel")) {
		channel = atoi(wert);
		channelset = true;
	    }
	    if (!strcmp(einstellung, "tcpport")) {
		tcpport = atoi(wert);
		portset = true;
	    }
	    if (!strcmp(einstellung, "fastforbackspeed")) {
		fastforbackspeed = atoi(wert);
		fastforbackspeedset = true;
	    }
	    if (!strcmp(einstellung, "mpddir")) {
		strcpy(winampPath, wert);
		xmmsdirset = true;
	    }
	    if (!strcmp(einstellung, "bookmarkfile")) {
		strcpy(bookmarkfile, wert);
		bookmarkset = true;
		bookmarkread = false;
	    }
/*	    if (!strcmp(einstellung, "autostartxmms")) {
		if (!strcmp(wert, "1"))
		    startAutomatically = 1;
		else
		    startAutomatically = 0;
		autoset = true;
	    }*/
/* I agree - ugly! */
nextConfigLine:;	    
	}
    }
    conffile.close();
    if (!(mp3set)) {
	Log("[bemusedMPD] Configfile is not written correctly\n");
	Log("[bemusedMPD] make sure that you set the mp3dir path (and others) in /etc/bemused.conf file!\n");
	exit(3);
    }
    if ((!channelset) && (!tcp)) {
	Log("[bemusedMPD] Warning: No channel defined in config file using default=10\n");
    }
    if ((!portset) && tcp) {
	Log("[bemusedMPD] Warning: No port defined in config file using default=7654\n");
    }
    // Read in bookmarks if we have them
    nobookmarks = 0;
    if (bookmarkset) {
	ifstream bookfile(bookmarkfile);
	if (!bookfile) {
	    Log("[bemusedMPD] Could not read the bookmarks file %s\n", bookmarkfile);
	    exit(2);
	}

	Log("[bemusedMPD] Read bookmarks file %s\n", bookmarkfile);
	while (bookfile.getline(row, 1024)) {
	    if (!(row[0] == '#') && !(row[0] == '\0') && (strchr(row, '=')!=NULL) ) {
		    
		    /* split up string, einstellung = wert */
		    tmpStr=strtok(row, "=");
		    if (tmpStr == NULL) {
			Log("[bemusedMPD] Invalid bookmark entry, Line: %s\n", row);
			goto nextBookmark;
		    }
		    strcpy(einstellung, tmpStr);
		    
		    tmpStr=strtok(NULL, "=");
		    if (tmpStr == NULL) {
			Log("[bemusedMPD] Invalid bookmark entry, Line: %s\n", row);
			goto nextBookmark;
		    }		    
		    strcpy(wert, tmpStr);

		    /* Trim any spaces out */
		    while (einstellung[strlen(einstellung) - 1] == ' ') {
			einstellung[strlen(einstellung) - 1] = '\0';
		    }
		    while (wert[strlen(wert) - 1] == ' ') {
			wert[strlen(wert) - 1] = '\0';
		    }
		    char altwert[1025];
		    while (wert[0] == ' ') {
			strcpy(altwert, &wert[1]);
			strcpy(wert, altwert);
		    }

		    if (strlen(wert) > MAX_STREAM_URL || strlen(wert)==0) {
			Log("[bemusedMPD] Bookmark URL '%s' is too large/small (> %i chars) - Skip it!\n", wert, MAX_STREAM_URL);
			goto nextBookmark;
		    }
		    if (strlen(einstellung) > MAX_STREAM_TITLE || strlen(einstellung)==0) {
			Log("[bemusedMPD] Bookmark Title '%s' is too large/Small (> %i chars) - Skip it!\n", einstellung, MAX_STREAM_TITLE);
			goto nextBookmark;			
		    }
#ifdef debuginf
		    Log("I have a bookmark \"%s\" of \"%s\"\n", einstellung, wert);
#endif
		    // Save this
		    strcpy(bookmarks[nobookmarks], wert);
		    strcpy(bookmarktitles[nobookmarks], einstellung);
		    nobookmarks++;
/* I agree - ugly! */
nextBookmark:;
	    }
	}
	bookfile.close();
	Log("[bemusedMPD] Found %i bookmarks\n", nobookmarks);
    }
    if (!fastforbackspeedset)
	fastforbackspeed = 10000;
}



int CBemusedServerDlg::HandleBluetooth()
{
    int exitApp;
    iExitNow = false;
    do {
	exitApp = ReadBluetoothCommand();
	if (exitApp==1)
	    return 1;
    } while (!iExitNow);

#ifdef debuginf
	Log("HandleBluetooth(), Close handle\n");
#endif

    close(iCommHandle);
}

bool CBemusedServerDlg::WriteFile(int handle, unsigned char *buffer, int maxbytes, long unsigned int *byteswritten, int *bla)
{
    long unsigned int b;
    int br;
    br = write(handle, (void *) buffer, maxbytes);
    if (br == -1) {
#ifdef debuginf
        Log("WriteFile1 failed write: <%s>\n", buffer);
#endif	     
	return false;
    } else {
	b = (long unsigned int) (br);
	*(byteswritten) = b;
	return true;
    }
}


bool CBemusedServerDlg::WriteFile(int handle, char *buffer, int maxbytes, long unsigned int *byteswritten, int *bla)
{
    long unsigned int b;
    int br;
    br = write(handle, (void *) buffer, maxbytes);
    if (br == -1) {
#ifdef debuginf
        Log("WriteFile2 failed write: <%s>\n", buffer);
#endif	     
	return false;
    } else {
	b = (long unsigned int) (br);
	*(byteswritten) = b;
	return true;
    }
}

bool CBemusedServerDlg::ReadFile(int handle, unsigned char *buffer, int maxbytes, long unsigned int *bytesread,
				 int *bla)
{
    long unsigned int b;
    int br;
    br = recv(handle, (void *) buffer, maxbytes, 0);
    if (br <= 0) {
#ifdef debuginf
        Log("ReadFile1 failed!\n");
#endif	     
	return false;
    } else {
	b = (long unsigned int) (br);
	*(bytesread) = b;
	return true;
    }

}


bool CBemusedServerDlg::ReadFile(int handle, char *buffer, int maxbytes, long unsigned int *bytesread, int *bla)
{
    long unsigned int b;
    int br;
    br = recv(handle, (void *) buffer, maxbytes, 0);
    if (br <= 0) {
#ifdef debuginf
        Log("ReadFile2 failed!\n");
#endif	     
	return false;
    } else {
	b = (long unsigned int) (br);
	*(bytesread) = b;
	return true;
    }

}

/* return 1 to exit application! */
int CBemusedServerDlg::ReadBluetoothCommand()
{
    int exitApp = 0;
    char buffer[5+16];
    unsigned long bytes_read = 0;

    memset(buffer, 0, 5+16);
    if (!ReadFile(iCommHandle, buffer, 4, &bytes_read, NULL)) {
	if (!tcp)
	    Log("[bemusedMPD] Couldn't read command: %s\n", GetLastError());
	else
	    Log("[bemusedMPD] Couldn't read command: Probably peer closed connection\n");
	iExitNow = true;
    } else {

	CheckWinamp("blub");
#ifdef debuginf
	Log("Command: <%s>, bytes read: %i\n", buffer,bytes_read);
#endif

	// List "neutral" commands first
	if (!strcmp(buffer, "LIST"))
	    WriteListToPhone();
	else if (!strcmp(buffer, "DLST"))
	    WriteDirectoryListToPhone();
	else if (!strcmp(buffer, "PLST"))
	    WritePlaylistToPhone();
	else if (!strcmp(buffer, "FINF"))
	    GetFileInfo();
	else if (!strcmp(buffer, "DOWN"))
	    DownloadFile();
	else if (!strcmp(buffer, "VOLM"))
	    SetVolume();
	else if (!strcmp(buffer, "CHCK"))
	    Check();
	else if (!strcmp(buffer, "VERS"))
	    Version();
	else if (!strcmp(buffer, "GVOL")) {
	    GetVolume();
	}
	else if (!strcmp(buffer, "INFO"))
	    WriteInfoToPhone();
	else if (!strcmp(buffer, "INF2"))
	    WriteInf2ToPhone();
	else if (!strcmp(buffer, "DINF"))
	    WriteDetailedInfoToPhone();
	else if (!strcmp(buffer, "RMAL"))
	    RemoveAllFromPlaylist();
	else {
	    // Put it in braces to avoid extraneous Log output
	    // Don't stop at end if any other command is executed
	    iStopEnd = false;

	    if (!strcmp(buffer, "PLAY"))
		PlayFile(false);
	    else if (!strcmp(buffer, "LADD"))
		PlayFile(true);
	    else if (!strcmp(buffer, "STOP"))
		Stop(true);
	    else if (!strcmp(buffer, "FADE"))
		FadeOut();
	    else if (!strcmp(buffer, "STEN")) {
		Log("[bemusedMPD] Will stop at start of next song\n");
		iStopEnd = true;
	    } 
	    else if (!strcmp(buffer, "PAUS")) {
		Pause();
	    }
	    else if (!strcmp(buffer, "STRT"))
		Play();
	    else if (!strcmp(buffer, "SLCT"))
		SelectInPlaylist();
	    else if (!strcmp(buffer, "PREV"))
		PreviousTrack();
	    else if (!strcmp(buffer, "NEXT"))
		NextTrack();
	    else if (!strcmp(buffer, "FFWD"))
		FastForward();
	    else if (!strcmp(buffer, "SEEK"))
		SetSeek();
	    else if (!strcmp(buffer, "RWND"))
		Rewind();
	    else if (!strcmp(buffer, "SHFL"))
		Shuffle();
	    else if (!strcmp(buffer, "REPT"))
		Repeat();
	    else if (!strcmp(buffer, "SHUT"))
		ShutDown();
	    else if (!strcmp(buffer, "EXIT"))
		//exit(0);
		exitApp=1;
	    else
		Log("[bemusedMPD] Read unknown command \"%s\"\n", buffer);
	}
    }
    return exitApp;
}

void CBemusedServerDlg::ReadDir(char *aDir, CDirTreeNode * aNode, bool aRecursive)
{
    char fileNameBuf[MAX_PATH + 1];
    sprintf(fileNameBuf, "%s/*.*", aDir);

#ifdef debuginf	
    Log("ReadDir: %s\n", fileNameBuf);
#endif

    DIR *dir;
    struct dirent *dirzeiger;
    struct stat attribut;
    int pathLen=strlen(aDir);
    
    //try open +Bookmarks folder
    if ((pathLen >=10) && !strcmp(aDir+(pathLen-10),"+Bookmarks"))
    {
#ifdef debuginf	
    Log("## try open +Bookmarks folder\n");
#endif    
       OpenBookmarkFolder(aDir, aNode, aRecursive);
       return;
    }
    

    if ((dir = opendir(aDir)) == NULL) {
	Log("[bemusedMPD] Couldn't open \"%s\"\n", aDir);
	return;
    }

    int foundCount = 0;
    CDirTreeNode *currentNode = aNode;

    while ((dirzeiger = readdir(dir)) != NULL) {
	if (!strcmp((*dirzeiger).d_name, ".") || !strcmp((*dirzeiger).d_name, ".."))
	    continue;

	if (foundCount == 0) {
	    aNode->iFirstChild = new CDirTreeNode;
	    aNode->iFirstChild->iParent = currentNode;
	    currentNode = aNode->iFirstChild;
	    currentNode->iNodeType = 0x1;	// child
	} else {
	    currentNode->iNextSibling = new CDirTreeNode;
	    currentNode->iNextSibling->iParent = currentNode->iParent;
	    currentNode = currentNode->iNextSibling;
	    currentNode->iNodeType = 0x3;	// sibling
	}

	foundCount++;

	sprintf(fileNameBuf, "%s/%s", aDir, (*dirzeiger).d_name);
	currentNode->iFileName = strdup((*dirzeiger).d_name);

	if (stat(fileNameBuf, &attribut) == -1)
	    Log("[bemusedMPD] ReadDir, error stat(): %s\n", GetLastError());
	if (S_ISDIR(attribut.st_mode))
	{
#ifdef debuginf	
	    Log("Directory Found: %s\n", fileNameBuf);
#endif
	    if (aRecursive) {
		currentNode->iFileType = 0x1;
		ReadDir(fileNameBuf, currentNode, true);
	    } else {
		currentNode->iFileType = 0x3;
	    }
	} else {
#ifdef debuginf	
	    Log("File Found: %s\n", fileNameBuf);
#endif	    
	    currentNode->iFileType = 0x2;
	}
    }
    
    if(nobookmarks > 0)
    {
        if (foundCount == 0)
        {
            aNode->iFirstChild = new CDirTreeNode;
            aNode->iFirstChild->iParent = currentNode;
            currentNode = aNode->iFirstChild;
            currentNode->iNodeType = 0x1; // child
        }
        else
	{
            currentNode->iNextSibling = new CDirTreeNode;
            currentNode->iNextSibling->iParent = currentNode->iParent;
            currentNode = currentNode->iNextSibling;
            currentNode->iNodeType = 0x3; // sibling
        }
	
     foundCount++;
     currentNode->iFileName = strdup("+Bookmarks");
     currentNode->iFileType = 0x3;
    }
    

    // if the last node was a child, make it an only child
    if (currentNode->iNodeType == 0x1)
	currentNode->iNodeType = 0x2;
    // if the last node was a sibling, make it the last sibling
    if (currentNode->iNodeType == 0x3)
	currentNode->iNodeType = 0x4;

#ifdef debuginf	
    Log("closedir..");
#endif	    
    closedir(dir);
#ifdef debuginf	
    Log(" done!\n");
#endif	    
    
}

// Looks for xmms, starting it if necessary. Returns whether
// xmms is running now

bool CBemusedServerDlg::CheckWinamp(const char *aWindowName)
{
#ifdef debuginf
    Log("check if mpd is running....\n");
#endif

    if (mpd_remote_is_running()) {
#ifdef debuginf
	Log("mpd IS running...\n");
#endif
	return true;
    }
#ifdef debuginf
    Log("WARNING: mpd is NOT running...\n");
#endif
    return false;
}

void CBemusedServerDlg::FadeOut()
{
    int curVol = mpd_remote_get_main_volume();
    int deltaVol = curVol / 10;	// 10*0.5s = 5 seconds
#ifdef debuginf    
    Log("curVol=%i\n", curVol);
#endif    
    for (int v = curVol; v > 0; v -= deltaVol) {
#ifdef debuginf    
	Log("Set volume to %i%\n", v);
#endif    	
	mpd_remote_set_main_volume(v);
	usleep(500000);		// 0.5 seconds
    }
    mpd_remote_set_main_volume(0);
#ifdef debuginf
    Log("Set volume to %i%\n", 0);
#endif        
    usleep(500000);
    Stop(true);
    sleep(1);			// Give time for Stop
#ifdef debuginf
    Log("Resetting volume to %i%\n", curVol);
#endif        
    mpd_remote_set_main_volume(curVol);
}


void CBemusedServerDlg::WriteDetailedInfoToPhone()
{
    // Our variables
    int bitrate = 0;
    int samplerate = 48;
    int nochannels = 1;
    int i;
    char buffer[MAX_PATH];

    // Get _current_ bit rate, sample rate and number of channels
    mpd_remote_get_info(&bitrate, &samplerate, &nochannels);

#ifdef debuginf
    Log("Got the song info, %d kbps, %d kHz, %d channels\n", bitrate, samplerate, nochannels);
#endif
    // Stuff this lot into buffer
    strcpy(buffer, "DINFACK");

    // Anyone think we ought to have a macro for this stuff?
    buffer[7] = bitrate >> 24;
    buffer[8] = (bitrate >> 16) & 0xFF;
    buffer[9] = (bitrate >> 8) & 0xFF;
    buffer[10] = bitrate & 0xFF;

    buffer[11] = samplerate >> 24;
    buffer[12] = (samplerate >> 16) & 0xFF;
    buffer[13] = (samplerate >> 8) & 0xFF;
    buffer[14] = samplerate & 0xFF;

    buffer[15] = nochannels >> 24;
    buffer[16] = (nochannels >> 16) & 0xFF;
    buffer[17] = (nochannels >> 8) & 0xFF;
    buffer[18] = nochannels & 0xFF;

    buffer[19] = '\0';

    long unsigned int bytesWritten;

    WriteFile(iCommHandle, buffer, (19), &bytesWritten, NULL);
    if (bytesWritten < 19)
	Log("[bemusedMPD] WriteDetailedInfoToPhone(): Only wrote %d bytes of DINFACK\n", bytesWritten);
#ifdef debuginf
    Log("Sent the Track Info to the phone\n");
#endif
}

void CBemusedServerDlg::WriteInfoToPhone()
{
    // Get the length of the song.
    int songLength;
    int count = 0;
    usleep(100000);
    do {
	songLength = mpd_remote_get_playlist_time(mpd_remote_get_playlist_pos());
	usleep(50);
	count++;
    } while ((songLength == -1) && (count < 100));

    if (songLength < 0)
	songLength = 0;

    unsigned char isPlaying = 0x0;
    if (mpd_remote_is_playing())
	isPlaying = 0x1;
    if (mpd_remote_is_paused())
	isPlaying = 0x2;


    int currentTime = mpd_remote_get_output_time();
    iStopNow = (iStopEnd && currentTime < lastTime);
    lastTime = currentTime;

    unsigned char repeat, shuffle;
    if (mpd_remote_is_repeat())
	repeat = 0x01;
    else
	repeat = 0x00;
    if (mpd_remote_is_shuffle())
	shuffle = 0x01;
    else
	shuffle = 0x00;


    char buffer[MAX_PATH];
    char title[255];
    unsigned int titlelength;

#ifdef debuginf
    Log("songLength1: %i\n", songLength);
#endif

    /* songlength of a stream is 0!*/
    strcpy(title, mpd_remote_get_playlist_title( mpd_remote_get_playlist_pos()) );
    titlelength = strlen(title);
    if (titlelength<1) {
	title[0] = ' ';
	title[1] = '\0';	
	titlelength = 1;    
    }
    
    strcpy(buffer, "INFOACK");
    buffer[7] = isPlaying;

    buffer[8] = songLength >> 24;
    buffer[9] = (songLength >> 16) & 0xFF;
    buffer[10] = (songLength >> 8) & 0xFF;
    buffer[11] = songLength & 0xFF;

    buffer[12] = currentTime >> 24;
    buffer[13] = (currentTime >> 16) & 0xFF;
    buffer[14] = (currentTime >> 8) & 0xFF;
    buffer[15] = currentTime & 0xFF;

    buffer[16] = shuffle;
    buffer[17] = repeat;
    strcpy(buffer + 18, title);

    long unsigned int bytesWritten;

    WriteFile(iCommHandle, buffer, (18 + titlelength), &bytesWritten, NULL);
    if (bytesWritten < (18 + titlelength))
	Log("[bemusedMPD] WriteInfoToPhone(): Only wrote %d bytes of INFOACK\n", bytesWritten);
    if (iStopEnd && iStopNow) {
	Stop(true);
	iStopEnd = iStopNow = false;
    }
}

void CBemusedServerDlg::WriteInf2ToPhone()
{
    // Get the length of the song.
    int songLength;
    int count = 0;
    do {
	songLength = mpd_remote_get_playlist_time( mpd_remote_get_playlist_pos() );
	usleep(50);
	count++;
    } while ((songLength == -1) && (count < 100));

    if (songLength < 0)
	songLength = 0;

    unsigned char isPlaying = 0x0;
    if (mpd_remote_is_playing())
	isPlaying = 0x1;
    if (mpd_remote_is_paused())
	isPlaying = 0x2;


    int currentTime = mpd_remote_get_output_time();
    iStopNow = (iStopEnd && currentTime < lastTime);
    lastTime = currentTime;

    unsigned char repeat, shuffle;
    if (mpd_remote_is_repeat())
	repeat = 0x01;
    else
	repeat = 0x00;
    if (mpd_remote_is_shuffle())
	shuffle = 0x01;
    else
	shuffle = 0x00;


    char buffer[MAX_PATH];
    char title[255];
    unsigned int titlelength;
    memset(buffer, 0, sizeof(buffer));
    
#ifdef debuginf
    Log("songLength2: %i\n", songLength);
#endif
    
    /* songlength of a stream is 0!*/
    strcpy(title, mpd_remote_get_playlist_title( mpd_remote_get_playlist_pos()) );
    titlelength = strlen(title);
    if (titlelength<1) {
	title[0] = ' ';
	title[1] = '\0';	
	titlelength = 1;    
    }

    strcpy(buffer, "INF2ACK");
    buffer[7] = isPlaying;

    buffer[8] = songLength >> 24;
    buffer[9] = (songLength >> 16) & 0xFF;
    buffer[10] = (songLength >> 8) & 0xFF;
    buffer[11] = songLength & 0xFF;

    buffer[12] = currentTime >> 24;
    buffer[13] = (currentTime >> 16) & 0xFF;
    buffer[14] = (currentTime >> 8) & 0xFF;
    buffer[15] = currentTime & 0xFF;

    buffer[16] = shuffle;
    buffer[17] = repeat;
    strcpy(buffer + 18, title);

    long unsigned int bytesWritten;

    WriteFile(iCommHandle, buffer, (19 + titlelength), &bytesWritten, NULL);
    if (bytesWritten < (19 + titlelength))
	Log("[bemusedMPD] WriteInf2ToPhone(): Only wrote %d bytes of INF2ACK\n", bytesWritten);
    if (iStopEnd && iStopNow) {
	Stop(true);
	iStopEnd = iStopNow = false;
    }
}

void CBemusedServerDlg::SetSeek()
{
    unsigned long bytesRead = 0;
    unsigned char indexBytes[4];
    int seek = 0;

    if (!ReadFile(iCommHandle, indexBytes, 4, &bytesRead, NULL)) {
	Log("[bemusedMPD] Couldn't read seek: %d\n", GetLastError());
	return;
    }

    seek = (indexBytes[0] << 24) + (indexBytes[1] << 16) + (indexBytes[2] << 8) + indexBytes[3];

    //mpd_remote_jump_to_time( seek * 1000);
    mpd_remote_jump_to_time( seek );
#ifdef debuginf
    Log("Set seek to %d seconds\n", seek);
#endif    
}

void CBemusedServerDlg::GetVolume()
{
//      unsigned char cVol=(unsigned char)((mpd_remote_get_main_volume(0)*256)/100);
// *256 seems buggy to me, it should be *255!
    char buffer[8];
#ifdef debuginf
    Log("get volume... ");
#endif
    int cVol = (mpd_remote_get_main_volume() * 255) / 100;
    
    if (cVol < 0)
	cVol = 0;
    if (cVol > 255)
	cVol = 255;
	
#ifdef debuginf
    Log("volume: %i\n", cVol);
#endif

    strcpy(buffer, "GVOLACK");
    buffer[7] = cVol & 0xFF;

    long unsigned int bytesWritten;
    WriteFile(iCommHandle, buffer, 8, &bytesWritten, NULL);

#ifdef debuginf
    Log("Send Volume\n");
#endif    

}

void CBemusedServerDlg::Version()
{
    char buffer[9];
    strcpy(buffer, "VERSACK");
    buffer[7] = char (1);
    buffer[8] = char (74);
    long unsigned int bytesWritten;
    WriteFile(iCommHandle, buffer, 9, &bytesWritten, NULL);
#ifdef debuginf    
    Log("Send Version Info\n");
#endif
}

void CBemusedServerDlg::OpenBookmarkFolder(char* aDir, CDirTreeNode* aNode, bool aRecursive)
{
    int foundCount = 0;
    CDirTreeNode* currentNode = aNode;
#ifdef debuginf    
    Log("OpenBookmarkFolder, %i bookmarks\n", nobookmarks);
#endif

    for(int i=0;i<nobookmarks;i++)
    {
        
	if (foundCount == 0)
        {
            aNode->iFirstChild = new CDirTreeNode;
            aNode->iFirstChild->iParent = currentNode;
            currentNode = aNode->iFirstChild;
            currentNode->iNodeType = 0x1; // child
        }
        else
        {
            currentNode->iNextSibling = new CDirTreeNode;
            currentNode->iNextSibling->iParent = currentNode->iParent;
            currentNode = currentNode->iNextSibling;
            currentNode->iNodeType = 0x3; // sibling
        }
        
	foundCount++;
//	char m00[255];
//	memset(m00, 0, 255);
//	strcat(m00,"//");
//	strcat(m00,bookmarktitles[i]);
//	strcat(m00,".mp3");
	string bookname="//";
	bookname+=bookmarktitles[i];
	currentNode->iFileName = strdup(bookname.c_str());
//	currentNode->iFileName = strdup( m00 );

// filetype 1 = direcotry	display as directory
// filetype 2 = file		does not work
// filetype 3 = special?	display as directory
        currentNode->iFileType = 0x2; //2
	
#ifdef debuginf    
    Log("bookmark name: %s\n", currentNode->iFileName);
#endif
	
    }
    if (currentNode->iNodeType == 0x1)
        currentNode->iNodeType = 0x2;
    if (currentNode->iNodeType == 0x3)
        currentNode->iNodeType = 0x4;

}

void CBemusedServerDlg::RemoveAllFromPlaylist()
{
    mpd_remote_stop();
    mpd_remote_playlist_clear();
    Log("[bemusedMPD] Removed all songs from playlist\n");
}

void CBemusedServerDlg::SelectInPlaylist()
{
    unsigned char indexBytes[2];
    unsigned long bytesRead = 0;
    int pos;

    if (!ReadFile(iCommHandle, indexBytes, 2, &bytesRead, NULL)) {
	Log("[bemusedMPD] Couldn't read playlist selection: %d\n", GetLastError());
	return;
    }

    pos = (indexBytes[0] << 8) + indexBytes[1];
    mpd_remote_set_playlist_pos(pos);
#ifdef debuginf    
    Log("[bemusedMPD] Setting playlist position to %d\n", pos);
#endif
}


void CBemusedServerDlg::PlayFile(bool aAddToPlaylist)
{
    unsigned long bytes = 0;

    unsigned char ubuffer[10];
    char buffer[MAX_PATH];
    int filenameLength = 0;
    memset(buffer, 0, sizeof(buffer));

    if (!ReadFile(iCommHandle, ubuffer, 2, &bytes, NULL)) {
	Log("[bemusedMPD] PlayFile():Couldn't read filename length: %d\n", GetLastError());
	return;
    }

    filenameLength = (ubuffer[0] << 8) + ubuffer[1];

    if (!ReadFile(iCommHandle, buffer, filenameLength, &bytes, NULL)) {
	Log("[bemusedMPD] Couldn't read filename of length %d: %d\n", filenameLength, GetLastError());
	return;
    }

    for (int i = 0; i < filenameLength; i++)
	if (buffer[i] == '\\')
	    buffer[i] = '/';
#ifdef debuginf
    Log("[bemusedMPD] About to play file \"%s\"\n", buffer);
#endif
    // clear the playlist if required
    if (!aAddToPlaylist)
          mpd_remote_playlist_clear();

    // add this song to the playlist
    string bookName=buffer;
    
    //translate to url if bookmark
    unsigned long firstpos=bookName.find("+Bookmarks///");
    bool isBook = false;
    if(firstpos!=string::npos)
    {
        bookName=bookName.substr(firstpos+13); 
        int b_entry=0;
        for(;b_entry < nobookmarks ; b_entry++)
	{
	     string title=bookmarktitles[b_entry];
	     if(title==bookName)
	     {
	         strcpy(buffer,bookmarks[b_entry]);
		 isBook = true;
		 break;
	     }
	}    
    }    

    int bookInt;
    if (isBook) {
#ifdef debuginf    
	Log("song IS a bookmark (%s)\n", buffer);
#endif    	
	bookInt = 1;
    }
    else {
#ifdef debuginf        
	Log("song is NOT a bookmark\n");
#endif    
	bookInt = 0;
    }
        
    mpd_remote_playlist_add(buffer, bookInt, (char *)mp3Dirs);

    // start playing!
    if (!aAddToPlaylist) {
	Log("[bemusedMPD] Playing \"%s\"\n", buffer);
	mpd_remote_play();
    } else {
	Log("[bemusedMPD] Added \"%s\" to playlist\n", buffer);
	return;
    }

    int songLength;
    int count = 0;

    do {
	if (isBook) {
	    songLength = 0;
	    break;
	}
    
	songLength = mpd_remote_get_playlist_time( mpd_remote_get_playlist_pos() );
	usleep(800);
	count++;
    } while ((songLength == -1) && (count < 100));
}

void CBemusedServerDlg::GetFileInfo()
{
    unsigned long bytes = 0;
    unsigned char buffer[MAX_PATH];
    int filenameLength = 0;

    memset(buffer, 0, sizeof(buffer));

    if (!ReadFile(iCommHandle, buffer, 2, &bytes, NULL)) {
	Log("[bemusedMPD] Couldn't read filename length: %d\n", GetLastError());
	return;
    }

    filenameLength = (buffer[0] << 8) + buffer[1];
    buffer[0] = '\0';
    buffer[1] = '\0';

    if (!ReadFile(iCommHandle, buffer, filenameLength, &bytes, NULL)) {
	Log("[bemusedMPD] Couldn't read filename: %d\n", GetLastError());
	return;
    }

    for (int i = 0; i < filenameLength; i++)
	if (buffer[i] == '\\')
	    buffer[i] = '/';
#ifdef debuginf
    Log("Returning info about \"%s\": ", buffer);
#endif
    FILE *file = fopen((char *) buffer, "rb");

    if (file) {
	fseek(file, 0, SEEK_END);
	long fileLength = ftell(file);
	fseek(file, 0, SEEK_SET);
	fclose(file);
#ifdef debuginf
	Log("length is %d\n", fileLength);
#endif
	char ackBuf[11];
	strcpy(ackBuf, "FINFACK");
	ackBuf[7] = (fileLength >> 24) & 0xFF;
	ackBuf[8] = (fileLength >> 16) & 0xFF;
	ackBuf[9] = (fileLength >> 8) & 0xFF;
	ackBuf[10] = fileLength & 0xFF;

	WriteFile(iCommHandle, ackBuf, 11, &bytes, NULL);
    } else {
	Log("[bemusedMPD] GetFileInfo(): couldn't open file\n");
    }

}


void CBemusedServerDlg::DownloadFile()
{
    unsigned long bytes = 0;

    unsigned char buffer[MAX_PATH];
    int filenameLength = 0;
    memset(buffer, 0, sizeof(buffer));

    if (!ReadFile(iCommHandle, buffer, 2, &bytes, NULL)) {
	Log("[bemusedMPD] DownloadFile(): Couldn't read filename length: %d\n", GetLastError());
	return;
    }

    filenameLength = (buffer[0] << 8) + buffer[1];
    buffer[0] = '\0';
    buffer[1] = '\0';

    if (!ReadFile(iCommHandle, buffer, filenameLength, &bytes, NULL)) {
	Log("[bemusedMPD] DownloadFile(): Couldn't read filename: %d\n", GetLastError());
	return;
    }

    for (int i = 0; i < filenameLength; i++)
	if (buffer[i] == '\\')
	    buffer[i] = '/';

    Log("[bemusedMPD] Uploading \"%s\"... ", buffer);

    FILE *file = fopen((char *) buffer, "rb");
    
    if (file == NULL) {
        Log("[bemusedMPD] ERROR, couldn't open file %s\n", buffer);
        return;
    }
			    
    unsigned long bytesRead;


    char ackBuf[8];
    strcpy(ackBuf, "DOWNACK");
    WriteFile(iCommHandle, ackBuf, 7, &bytes, NULL);

    do {
        bytesRead = fread(buffer, 1, sizeof(buffer), file);

        if (bytesRead > 0) {
	    WriteFile(iCommHandle, buffer, bytesRead, &bytes, NULL);
//                              if (bytes < bytesRead)
//                                      Log("wrote %d bytes of %d read from file\n", bytes, bytesRead);
	}
    } while (bytesRead > 0);

#ifdef debuginf
    Log("finished\n");
    Log("close file...");
#endif	
    
    fclose(file);    
}

void CBemusedServerDlg::SetVolume()
{
    unsigned long bytesRead = 0;
    unsigned char volume = 0;

    if (!ReadFile(iCommHandle, &volume, 1, &bytesRead, NULL)) {
	Log("[bemusedMPD] Couldn't read volume: %d\n", GetLastError());
	return;
    }
    // set the volume

    mpd_remote_set_main_volume((volume * 100) / 255);
#ifdef debuginf
    Log("Set volume to %d%%\n", (volume * 100) / 255);
#endif    
}

void CBemusedServerDlg::Play()
{
    if (mpd_remote_get_playlist_length())	// to prevent open file dialog opening
    {
	mpd_remote_play();
#ifdef debuginf	
	Log("Playing\n");
#endif	
    } else {
	Log("[bemusedMPD] Received PLAY, but there is nothing to play\n");
    }
}

void CBemusedServerDlg::Stop(bool aFadeOut)
{
    if (aFadeOut)
	mpd_remote_stop();
    else
	mpd_remote_stop();
#ifdef debuginf
    Log("Stopping\n");
#endif    
}

void CBemusedServerDlg::Pause()
{
    mpd_remote_pause();
#ifdef debuginf    
    Log("Pause\n");
#endif    
}

void CBemusedServerDlg::NextTrack()
{
    mpd_remote_playlist_next();
#ifdef debuginf    
    Log("Next track\n");
#endif
    int songLength;
    int count = 0;

    do {
	songLength = mpd_remote_get_playlist_time( mpd_remote_get_playlist_pos() );

	usleep(50);
	count++;
    } while ((songLength == -1) && (count < 100));
}

void CBemusedServerDlg::PreviousTrack()
{
    mpd_remote_playlist_prev();
#ifdef debuginf    
    Log("Previous track\n");
#endif
    int songLength;
    int count = 0;

    do {
	songLength = mpd_remote_get_playlist_time( mpd_remote_get_playlist_pos() );
	usleep(50);
	count++;
    } while ((songLength == -1) && (count < 100));
}

void CBemusedServerDlg::FastForward()
{
    int atime = mpd_remote_get_output_time();
    mpd_remote_jump_to_time(atime + fastforbackspeed);
#ifdef debuginf    
    Log("Fast forward\n");
#endif
}

void CBemusedServerDlg::Check()
{
    long unsigned int bytesWritten;
    char check[] = "Y";
    WriteFile(iCommHandle, check, 1, &bytesWritten, NULL);
}


void CBemusedServerDlg::Rewind()
{
    int atime = mpd_remote_get_output_time();
    atime -= fastforbackspeed;
    if (atime < 0)
	atime = 0;
    mpd_remote_jump_to_time(atime);
#ifdef debuginf    
    Log("Rewind\n");
#endif    
}


void CBemusedServerDlg::Shuffle()
{
    unsigned long bytesRead = 0;
    unsigned char shuffleOn = 0;

    if (!ReadFile(iCommHandle, &shuffleOn, 1, &bytesRead, NULL)) {
	Log("[bemusedMPD] Couldn't read shuffle: %d\n", GetLastError());
	return;
    }
    mpd_remote_toggle_shuffle();
#ifdef debuginf
    Log("Changing shuffle to %s\n", shuffleOn ? "on" : "off");
#endif    
}


void CBemusedServerDlg::Repeat()
{
    unsigned long bytesRead = 0;
    unsigned char repeatOn = 0;

    if (!ReadFile(iCommHandle, &repeatOn, 1, &bytesRead, NULL)) {
	Log("[bemusedMPD] Couldn't read repeat: %d\n", GetLastError());
	return;
    }

    mpd_remote_toggle_repeat();
#ifdef debuginf
    Log("Changing repeat to %s\n", repeatOn ? "on" : "off");
#endif    
}

void CBemusedServerDlg::ShutDown()
{
    Log("[bemusedMPD] Shutting down Linux (only, if you have the right to do so)\n");
    system("/sbin/shutdown -h now");
}

void CBemusedServerDlg::ResetWrittenFlag(CDirTreeNode * aNode)
{
    aNode->iWrittenToBuffer = false;

    if (aNode->iFirstChild)
	ResetWrittenFlag(aNode->iFirstChild);
    if (aNode->iNextSibling)
	ResetWrittenFlag(aNode->iNextSibling);
}



void CBemusedServerDlg::WriteListToPhone()
{
    unsigned long bytesWritten = 0;
    unsigned char buffer[MAX_PATH];
    unsigned char *bufferPtr;

    // Get a list of the files we can browse.
    char pathBuffer[MAX_PATH + 1];

    strcpy((char *) pathBuffer, mp3Dirs);
    if ((strlen(pathBuffer) > 0) && (pathBuffer[strlen(pathBuffer) - 1] == '/'))
	pathBuffer[strlen(pathBuffer) - 1] = '\0';


    GenerateFileList(pathBuffer, true);

    WriteFile(iCommHandle, "LISTACK", 7, &bytesWritten, NULL);
    if (bytesWritten < 7) {
	Log("[bemusedMPD] Couldn't write list to phone (error %d)\n", GetLastError());
	return;
    }

    int sizeLeft;
    int nextPos;
    bool allDone;

    int totalBytesWritten = 0;

    // If we don't have any files, just return end-of-list.
    if (!iDirTreeRoot) {
	buffer[0] = 0xFF;
	buffer[1] = 0x00;
	WriteFile(iCommHandle, buffer, 2, &bytesWritten, NULL);
	return;
    }

    iDirTreeCurrentNode = iDirTreeRoot;
    ResetWrittenFlag(iDirTreeRoot);

    do {
	sizeLeft = sizeof(buffer) - 2;

	bufferPtr = buffer;
	memset(buffer, 0, sizeof(buffer));
	allDone = WriteNodeToBuffer(iDirTreeRoot, bufferPtr, sizeLeft);

	totalBytesWritten += (sizeof(buffer) - sizeLeft);
	nextPos = sizeof(buffer) - 2 - sizeLeft;

	if (allDone) {
	    // Write "last entry" token.
	    buffer[nextPos++] = 0xFF;
	    buffer[nextPos++] = 0x00;
	}

	bytesWritten = 0;
	WriteFile(iCommHandle, buffer, nextPos, &bytesWritten, NULL);
	if (bytesWritten < (unsigned long) nextPos) {
	    Log("[bemusedMPD] Couldn't write list to phone (error %d)\n", GetLastError());
	    return;
	}
    } while (!allDone);

    Log("[bemusedMPD] WriteListToPhone(): Wrote song list to phone (%d bytes)\n", totalBytesWritten);
}


void CBemusedServerDlg::WritePlaylistToPhone()
{
    unsigned long bytesWritten = 0;
    char buffer[MAX_PATH];
    char eol[2];
    int i;
    int towrite = 0;
    int noentries = 0;
    int playlistpos;

    eol[0] = '\n';
    eol[1] = '\0';

    // Fetch details
    playlistpos = mpd_remote_get_playlist_pos();
#ifdef debuginf    
    Log("Got playlist pos: %i\n",playlistpos);
#endif    
    noentries = mpd_remote_get_playlist_length();
#ifdef debuginf    
    Log("Entries %d\n", noentries);
#endif    
    // Stuff this lot into buffer
    strcpy(buffer, "PLSTACK");
    buffer[7] = (playlistpos >> 8) & 0xFF;
    buffer[8] = playlistpos & 0xFF;

    buffer[9] = '#';
    buffer[10] = '\n';		// need some junk and a \n after the size
    // winamp achieves this with the m3u header
    towrite = 11;

    // Send this bit on it's way
    WriteFile(iCommHandle, buffer, towrite, &bytesWritten, NULL);
    if (bytesWritten < (unsigned) towrite)
	Log("[bemusedMPD] Only wrote %d bytes of PLSTACK\n", bytesWritten);

    // Now pull down and send the playlist
    for (i = 0; i < noentries; i++) {
	sprintf(buffer, mpd_remote_get_playlist_title(i));
#ifdef debuginf	
	Log("Track Title is \"%s\"\n", buffer);
#endif	
	strcat(buffer, eol);	// add \n
	towrite = strlen(buffer);

	WriteFile(iCommHandle, buffer, towrite, &bytesWritten, NULL);
	if (bytesWritten < (unsigned) towrite)
	    Log("[bemusedMPD] Only wrote %d bytes of PLSTACK\n", bytesWritten);
    }

    // Send the end
    buffer[0] = '\0';
    WriteFile(iCommHandle, buffer, 1, &bytesWritten, NULL);

    Log("[bemusedMPD] Sent the playlist to the phone, %d entries, position is %d\n", noentries, playlistpos);

}


void CBemusedServerDlg::WriteDirectoryListToPhone()
{
    unsigned long bytesWritten = 0;
    unsigned char buffer[MAX_PATH];
    unsigned char *bufferPtr;

#ifdef debuginf	
    Log("WriteDirectoryListToPhone\n");
#endif	

    int filenameLength = 0;
    unsigned long bytes;
    memset(buffer, 0, sizeof(buffer));

    if (!ReadFile(iCommHandle, buffer, 2, &bytes, NULL)) {
	Log("[bemusedMPD] Couldn't read directory name length: %d\n", GetLastError());
	return;
    }

    filenameLength = (buffer[0] << 8) + buffer[1];
    buffer[0] = '\0';
    buffer[1] = '\0';

#ifdef debuginf	
    Log("Filenamelength: %i\n", filenameLength);
    Log("Buffer: %s\n", buffer);
#endif	

    if ((filenameLength != 0) && (!ReadFile(iCommHandle, buffer, filenameLength, &bytes, NULL))) {
	Log("[bemusedMPD] Couldn't read directory name: %d\n", GetLastError());
	return;
    }

#ifdef debuginf	
    Log("process path...\n");
#endif	

    for (int i = 0; i < filenameLength; i++) {
	if (buffer[i] == '\\')
	    buffer[i] = '/';
    }

#ifdef debuginf	
    Log("find null... mp3dirs: %i (%s)\n",strlen(mp3Dirs), mp3Dirs);
#endif	

    if (!strcmp((char *) buffer, "")) {
	strcpy((char *) buffer, mp3Dirs);
	if ((strlen((char *) buffer) > 0) && (buffer[strlen((char *) buffer) - 1] == '/'))
	    buffer[strlen((char *) buffer) - 1] = '\0';
    }
    //buffer[filenameLength]='/';

    GenerateFileList((char *) buffer, false);

    // Note that we return a standard LISTACK to simplify client-side handling.
    WriteFile(iCommHandle, "LISTACK", 7, &bytesWritten, NULL);
    if (bytesWritten < 7) {
	Log("[bemusedMPD] Couldn't write list to phone (error %d)\n", GetLastError());
	return;
    }

    int sizeLeft;
    int nextPos;
    bool allDone;

    int totalBytesWritten = 0;

    // If we don't have any files, just return end-of-list.
    if (!iDirTreeRoot) {
	buffer[0] = 0xFF;
	buffer[1] = 0x00;
	WriteFile(iCommHandle, buffer, 2, &bytesWritten, NULL);

	Log("[bemusedMPD] no files to list\n");
	return;
    }

    iDirTreeCurrentNode = iDirTreeRoot;
    ResetWrittenFlag(iDirTreeRoot);

    do {
	sizeLeft = sizeof(buffer) - 2;

	bufferPtr = buffer;
	memset(buffer, 0, sizeof(buffer));
	allDone = WriteNodeToBuffer(iDirTreeRoot, bufferPtr, sizeLeft);

	totalBytesWritten += (sizeof(buffer) - sizeLeft);
	nextPos = sizeof(buffer) - 2 - sizeLeft;

	if (allDone) {
	    // Write "last entry" token.
	    buffer[nextPos++] = 0xFF;
	    buffer[nextPos++] = 0x00;
	}

	bytesWritten = 0;
	WriteFile(iCommHandle, buffer, nextPos, &bytesWritten, NULL);

	if (bytesWritten < (unsigned long) nextPos) {
	    Log("[bemusedMPD] Couldn't write list to phone (error %d)\n", GetLastError());
	    return;
	}
    } while (!allDone);

    Log("[bemusedMPD] WriteDirectoryListToPhone(): wrote list to phone (%d bytes)\n", totalBytesWritten);
}




bool CBemusedServerDlg::WriteNodeToBuffer(CDirTreeNode * aNode, unsigned char *&aBufferPtr, int &aSizeRemaining)
{
    int lengthNeeded = strlen(aNode->iFileName) + 2;
    if (aSizeRemaining < lengthNeeded)
	return false;

    unsigned char type = (aNode->iNodeType << 4) + aNode->iFileType;

    if (!aNode->iWrittenToBuffer) {
	aBufferPtr[0] = type;
	memcpy(&aBufferPtr[1], aNode->iFileName, lengthNeeded - 1);

	aBufferPtr += lengthNeeded;
	aSizeRemaining -= lengthNeeded;

	aNode->iWrittenToBuffer = true;
    }

    int ret = true;

    if (ret && aNode->iFirstChild) {
//              iDirTreeCurrentNode = aNode->iFirstChild;
	ret = WriteNodeToBuffer(aNode->iFirstChild, aBufferPtr, aSizeRemaining);
    }

    if (ret) {
	if (aNode->iNextSibling) {
//                      iDirTreeCurrentNode = aNode->iNextSibling;
	    ret = WriteNodeToBuffer(aNode->iNextSibling, aBufferPtr, aSizeRemaining);
	}
//              else
//                      {
//                      if (aNode->iParent && aNode->iParent->iNextSibling)
//                              {
//                              iDirTreeCurrentNode = aNode->iParent->iNextSibling;
//                              ret = WriteNodeToBuffer(iDirTreeCurrentNode, aBufferPtr, aSizeRemaining);
//                              }
//                      }
    }
//      if (aNode->iFirstChild)
//              iDirTreeCurrentNode = aNode->iFirstChild;

    return ret;
}






void CBemusedServerDlg::GenerateFileList(const char *aRoot, bool aRecursive)
{
//    delete iDirTreeRoot;
    if (iDirTreeRoot)
       delete iDirTreeRoot;
        // RecursiveDelete(iDirTreeRoot);

    iDirTreeRoot = NULL;

#ifdef debuginf
    Log("GenerateFileList\n");
#endif

    if (!strcmp(aRoot, ""))
	return;

    iDirTreeRoot = new CDirTreeNode;
    iDirTreeRoot->iFileType = 0x1;	// dir
    iDirTreeRoot->iNodeType = 0x0;	// root
    iDirTreeRoot->iFirstChild = iDirTreeRoot->iNextSibling = NULL;
    iDirTreeCurrentNode = iDirTreeRoot;

#ifdef debuginf
    Log("-process dir %s...\n",aRoot);
#endif

    // Different handling here based on whether we've only got one directory as our root,
    // or multiple directories.
    if (strchr(aRoot, ';')) {
	iDirTreeRoot->iFileName = strdup("");

	char rootPath[MAX_PATH];
	const char *rootPtr = aRoot;
	CDirTreeNode *newNode;
	CDirTreeNode *lastNewNode=NULL;
	int pathCount = 0;
	bool onLastPath = false;

	while (1) {
	    memset(rootPath, 0, sizeof(rootPath));

	    if (!onLastPath) {
		strncpy(rootPath, rootPtr, strchr(rootPtr, ';') - rootPtr);
		rootPtr = strchr(rootPtr, ';') + 1;
	    } else {
		strcpy(rootPath, rootPtr);
	    }
#ifdef debuginf
    Log("rootPath: \n", rootPath);
#endif

	    // Remove a trailing backslash if there is one.
	    if ((strlen(rootPath) > 0) && (rootPath[strlen(rootPath) - 1] == '/'))
		rootPath[strlen(rootPath) - 1] = '\0';

	    // Check that we've got a valid path.
	    if (!strcmp(rootPath, "")) {
		if (!strchr(rootPtr, ';'))
		    onLastPath = true;
		continue;
	    }

	    pathCount++;

	    newNode = new CDirTreeNode;
	    newNode->iParent = iDirTreeRoot;
	    newNode->iFileName = strdup(rootPath);

	    if (aRecursive)
		newNode->iFileType = 0x1;	// dir
	    else
		newNode->iFileType = 0x3;	// unexpanded dir

	    if (pathCount == 1) {
		iDirTreeRoot->iFirstChild = newNode;
		newNode->iNodeType = 0x1;	// child
	    } else {
		lastNewNode->iNextSibling = newNode;
		newNode->iNodeType = 0x3;	// sibling
	    }

	    iDirTreeCurrentNode = newNode;

	    if (aRecursive)
		ReadDir(newNode->iFileName, newNode, aRecursive);

	    lastNewNode = newNode;

	    if (onLastPath)
		break;

	    if (!strchr(rootPtr, ';'))
		onLastPath = true;
	}
#ifdef debuginf	
    Log("bookmarkcheck\n");
#endif

	// Bookmarks
	if (nobookmarks > 0) {
            newNode = new CDirTreeNode;
            newNode->iParent = iDirTreeRoot;
	    newNode->iFileName = strdup("+Bookmarks");
	    newNode->iFileType = 0x3;
            lastNewNode->iNextSibling = newNode;
            newNode->iNodeType = 0x3; // sibling
	    iDirTreeCurrentNode = newNode;
	    lastNewNode = newNode;	
	} else if (newNode)
	    newNode->iNodeType = 0x4;	// last sibling
    } else {
	iDirTreeRoot->iFileName = strdup(aRoot);
#ifdef debuginf	
    Log("else loop\n");
#endif

	ReadDir(iDirTreeRoot->iFileName, iDirTreeRoot, aRecursive);
    }
}


int CBemusedServerDlg::BluetoothConnectToSocket(int channel)
{
    int s = 0;

    struct sockaddr_rc loc_addr;
    struct sockaddr_in iploc_addr;
    int pf;
    int proto;

    pf = PF_BLUETOOTH;
    proto = BTPROTO_RFCOMM;

    if (tcp) {
	pf = PF_INET;
	proto = IPPROTO_TCP;
    }

    if ((s = socket(pf, SOCK_STREAM, proto)) < 0) {
	Log("[bemusedMPD] Can't create socket. %s\n. Exit now...", strerror(errno));
	exit(1);
    }
    loc_addr.rc_family = AF_BLUETOOTH;
    loc_addr.rc_bdaddr = bdaddr;
    loc_addr.rc_channel = uint8_t(channel);
    if (!tcp) {
	char tmpstring[50];
	Log("[bemusedMPD] execute \"sdptool add --channel=%i SP\":\n", channel);
	sprintf(tmpstring, "sdptool add --channel=%i SP", channel);
	system(tmpstring);
	Log("[bemusedMPD] registered SP for channel %i\n", channel);
    }
    if (tcp) {
	iploc_addr.sin_family = AF_INET;
	iploc_addr.sin_port = htons(tcpport);
	struct in_addr lokaleip;
	lokaleip.s_addr = INADDR_ANY;
	iploc_addr.sin_addr = lokaleip;
    }
    struct sockaddr *socketaddr = (struct sockaddr *) &loc_addr;
    if (tcp)
	socketaddr = (struct sockaddr *) &iploc_addr;

    if (bind(s, socketaddr, sizeof(*socketaddr)) < 0) {
	Log("[bemusedMPD] Can't bind socket, Error \"%s\". Exit now...\n", strerror(errno));
	exit(1);
    }

    if (listen(s, 10)) {
	Log("[bemusedMPD] Can not listen on the socket. %s. Exit now...\n", strerror(errno));
	exit(1);
    }
    return s;

}


int CBemusedServerDlg::BluetoothWaitforConnect(int s)
{
    int s1 = 0, opt;
    bdaddr_t ba;
    struct sockaddr_rc rem_addr;
    struct sockaddr_in iprem_addr;

    if (!tcp)
	Log("[bemusedMPD] Waiting for connection over bluetooth\n");
    else
	Log("[bemusedMPD] Waiting for conection over TCP/IP (Port: %i)\n", tcpport);

    struct sockaddr *socketaddr = (struct sockaddr *) &rem_addr;
    if (!tcp) {
	opt = sizeof(rem_addr);
    } else {
	opt = sizeof(iprem_addr);
	socketaddr = (struct sockaddr *) &iprem_addr;
    }
    if ((s1 = accept(s, socketaddr, (socklen_t *) & opt)) < 0) {
	Log("[bemusedMPD] Accept failed. %s\n", strerror(errno));
	exit(1);
    }

    if (!tcp) {
	baswap(&ba, &rem_addr.rc_bdaddr);
	Log("[bemusedMPD] Connect from %s\n", batostr(&ba));
    } else {
	Log("[bemusedMPD] Connect from %s\n", inet_ntoa(iprem_addr.sin_addr));
    }

    return s1;
}


void CBemusedServerDlg::Log(char *aFormatString, ...)
{
    char logBuffer[MAX_PATH];
    va_list argList;
    va_start(argList, aFormatString);
    vsprintf(logBuffer, aFormatString, argList);
    va_end(argList);
    cout << logBuffer;
}


char *CBemusedServerDlg::GetLastError()
{
    return strerror(errno);
}

/*
void CBemusedServerDlg::RecursiveDelete(CDirTreeNode* node)
{
     if(node == NULL)
        return;
     if(node->iFirstChild)
         RecursiveDelete(node->iFirstChild);
     if(node->iNextSibling)
         RecursiveDelete(node->iNextSibling);
     delete node;
}
*/
