/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2003 Ashley Montanaro and Daniel Winter.
*/


#define MAX_PATH 1024
#define BOOKMARK_COUNT 100
#define MAX_STREAM_URL 255
#define MAX_STREAM_TITLE 100

/////////////////////////////////////////////////////////////////////////////
// CDirTreeNode class

class CDirTreeNode {
  public:
    CDirTreeNode();
    ~CDirTreeNode();

    unsigned char iNodeType;
    unsigned char iFileType;
    char *iFileName;

    bool iWrittenToBuffer;

    CDirTreeNode *iParent;
    CDirTreeNode *iFirstChild;
    CDirTreeNode *iNextSibling;
};


/////////////////////////////////////////////////////////////////////////////
// CBemusedServerDlg dialog

class CBemusedServerDlg {
// Construction
  public:
    CBemusedServerDlg();	// standard constructor
    ~CBemusedServerDlg();
    bool OnInitDialog(bool daemonize = false);

// Implementation

  private:
    int BluetoothConnectToSocket(int Channel);
    int BluetoothWaitforConnect(int s);
    int ReadBluetoothCommand();
    int HandleBluetooth();

    bool CheckWinamp(const char *aWindowName);
    bool ReadFile(int handle, char *buffer, int maxbytes, long unsigned int *bytesread, int *bla);
    bool ReadFile(int handle, unsigned char *buffer, int maxbytes, long unsigned int *bytesread, int *bla);
    bool WriteFile(int handle, char *buffer, int maxbytes, long unsigned int *byteswritten, int *bla);
    bool WriteFile(int handle, unsigned char *buffer, int maxbytes, long unsigned int *byteswritten, int *bla);
    char *GetLastError();

    void WriteListToPhone();
    void WriteDirectoryListToPhone();
    void WritePlaylistToPhone();

    void WriteInfoToPhone();
    void WriteInf2ToPhone();
    void WriteDetailedInfoToPhone();
    void Play();
    void PlayFile(bool aAddToPlaylist);
    void GetFileInfo();
    void SetSeek();
    void DownloadFile();
    void SetVolume();
    void GetVolume();
    void Version();
    void SelectInPlaylist();
    void RemoveAllFromPlaylist();
    void Stop(bool aFadeOut);
    void Pause();
    void NextTrack();
    void PreviousTrack();
    void FastForward();
    void Rewind();
    void Shuffle();
    void Repeat();
    void ShutDown();
    void Check();
    void FadeOut();
    void ReadIniFile();

    void RecursiveDelete(CDirTreeNode* node);
    void GenerateFileList(const char *aRoot, bool aRecursive);
    void OpenBookmarkFolder(char* aDir, CDirTreeNode* aNode, bool aRecursive);
    void ReadDir(char *aDir, CDirTreeNode * aNode, bool aRecursive);
    bool WriteNodeToBuffer(CDirTreeNode * aNode, unsigned char *&aBufferPtr, int &aSizeRemaining);
    void ResetWrittenFlag(CDirTreeNode * aNode);

    void Log(char *aFormatString, ...);

    CDirTreeNode *iDirTreeRoot;
    CDirTreeNode *iDirTreeCurrentNode;

    int iCommHandle;
    bool iExitNow;
    bool iStopEnd;
    bool iStopNow;
//    int startAutomatically;
    char winampPath[MAX_PATH];
    char bookmarkfile[MAX_PATH];
    char bookmarks[BOOKMARK_COUNT][MAX_STREAM_URL];		// stream URLS
    char bookmarktitles[BOOKMARK_COUNT][MAX_STREAM_TITLE];	// stream Titles
    int nobookmarks;
    int socket1;
    int channel;
    int fastforbackspeed;
    char mp3Dirs[MAX_PATH];
    bool tcp;
    int tcpport;
    int lastTime;
};
